/* gcompris - shapegame.c
 *
 * Time-stamp: <2001/03/26 00:02:37 bruno>
 *
 * Copyright (C) 2000 Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

/* libxml includes */
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include <math.h>

#include "gcompris-board.h"
#include "gameutil.h"

#define SOUNDLISTFILE PACKAGE

static GList *item_list = NULL;

static GcomprisGame *gcomprisGame = NULL;

typedef enum
{
  SHAPE_TARGET		= 1 << 0,
  SHAPE_ICON		= 1 << 1,
  SHAPE_BACKGROUND	= 1 << 2
} ShapeType;

/* Let's define the structure for a single shape */
typedef struct _Shape Shape;
struct _Shape {
  char  *name;			/* name of the shape */
  char  *pixmapfile;		/* relative pixmap file name of the shape */
  double x;			/* x coordinate */
  double y;			/* y coordinate */
  double zoomx;			/* x zoom factor */
  double zoomy;			/* y zoom factor */
  gint   position;		/* depth position 0=bottom other=intermediate */
  ShapeType type;		/* Type of shape */

  GnomeCanvasItem *item;       	/* Canvas item for this shape */
  Shape *icon_shape;		/* Temporary Canvas icon shape for this shape */
  GnomeCanvasItem *bad_item;	/* Temporary bad placed Canvas item for this shape */

  gboolean found;		/* The user found this item */
};

/* This is the list of shape for the current game */
static GList *shape_list = NULL;

/* Hash table of all the shapes in the list of shapes (one by different pixmap plus icon list) */
static GHashTable *shapelist_table = NULL;
static gint SHAPE_BOX_WIDTH_RATIO = 14;

static GnomeCanvasItem *shape_root_item;
static GnomeCanvasItem *shape_list_root_item;

static GnomeCanvasItem *shapegame_init_canvas(GnomeCanvasGroup *parent);
static void shapegame_destroy_all_items(void);
static void setup_item(GnomeCanvasItem *item, Shape *shape);
static void shapegame_next_level(void);
static gboolean read_xml_file(char *fname);
static gboolean write_xml_file(char *fname);
static Shape *find_closest_shape(double x, double y, double limit);
static Shape *create_shape(ShapeType type, char *name, char *pixmapfile, 
			   double x, double y, double zoomx, 
			   double zoomy, guint position);
static void process_ok();

/*
 * Main entry point mandatory for each Gcompris's game
 * ---------------------------------------------------
 *
 */

/*
 * in : boolean TRUE = PAUSE : FALSE = CONTINUE
 *
 */
static void shapegame_pause_game (gboolean pause)
{

  if(gcomprisGame==NULL)
    return;

}

/*
 * The opt param is unused
 */
static void shapegame_start_game (GnomeCanvas *theCanvas,
				  char *opt)
{

  if(gcomprisGame!=NULL)
    {
      gcomprisGame->canvas = theCanvas;

      set_background(gnome_canvas_root(gcomprisGame->canvas), "gcompris-bg.jpg");

      set_game_size(theCanvas, gcomprisGame);

      /* set initial values for this level */
      gcomprisGame->level = 1;

      gcomprisGame->sublevel = 0;
      bar_set_timer(gcomprisGame->sublevel);

      gcomprisGame->number_of_sublevel=1;
      bar_set_maxtimer(gcomprisGame->number_of_sublevel);

      shapegame_next_level();

      shapegame_pause_game(FALSE);

    }

}

static void
shapegame_end_game ()
{

  if(gcomprisGame!=NULL)
    {
      shapegame_pause_game(TRUE);
      shapegame_destroy_all_items();
      gcomprisGame->level = 1;       // Restart this game to zero
    }
}

static void
shapegame_free_game ()
{

  printf(">shapegame_end_game\n");

  if(gcomprisGame!=NULL)
    {
      shapegame_pause_game(TRUE);
      shapegame_destroy_all_items();
      g_free (gcomprisGame);
      gcomprisGame=NULL;
    }
}

static void
shapegame_set_level (guint level)
{

  printf(">shapegame_set_level\n");

  if(gcomprisGame!=NULL)
    {
      gcomprisGame->level=level;
      gcomprisGame->sublevel=0;
      shapegame_next_level();
    }
}

/*
 */
GcomprisGame *
shapegame_game_new ()
{

  /* This is a singleton */
  if(gcomprisGame!=NULL)
    return gcomprisGame;

  gcomprisGame= g_malloc (sizeof (GcomprisGame));

  if (gcomprisGame)
    {

      gcomprisGame->name=            _("Make the puzzle");
      gcomprisGame->description=     _("Drag and Drop the items to rebuild the original paintings");
      gcomprisGame->icon_name="gameicons/shapegame.png";
      gcomprisGame->maxlevel = 6;

      gcomprisGame->pause_game=	&shapegame_pause_game;
      gcomprisGame->start_game=	&shapegame_start_game;
      gcomprisGame->end_game=	&shapegame_end_game;
      gcomprisGame->free_game=	&shapegame_free_game;
      gcomprisGame->key_press=	NULL;      
      gcomprisGame->ok=		&process_ok;      
      gcomprisGame->set_level=	&shapegame_set_level;      

      gcomprisGame->number_of_sublevel=1; /* Go to next level after this number of 'play' */
    }

  return gcomprisGame;
}



/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------------*/

/* set initial values for the next level */
static void shapegame_next_level() 
{
  char *filename;

  bar_set_level(gcomprisGame);
  bar_set_timer(gcomprisGame->sublevel);

  shapegame_destroy_all_items();

  shapegame_init_canvas(gnome_canvas_root(gcomprisGame->canvas));

  filename = g_strdup_printf("%s%s%d_%d.xml",  
			     PACKAGE_DATA_DIR, "/shapesgame/board", 
			     gcomprisGame->level, gcomprisGame->sublevel);
  g_message("Trying to open file %s ", filename);
  read_xml_file(filename);

  g_free(filename);
}


static void process_ok()
{
  GList *list;
  gboolean done = TRUE;

  /* Loop through all the shapes to find if all target are found */
  for(list = shape_list; list != NULL; list = list->next) {
    Shape *shape = list->data;

    if(shape->type==SHAPE_TARGET)
      {
	if(shape->found==FALSE)
	  done=FALSE;
      }
  }

  if(done)
    {
      gcomprisGame->sublevel++;
      bar_set_timer(gcomprisGame->sublevel);

      if(gcomprisGame->sublevel>=gcomprisGame->number_of_sublevel) {
	/* Try the next level */
	gcomprisGame->level++;
	if(gcomprisGame->level>gcomprisGame->maxlevel)
	  gcomprisGame->level=gcomprisGame->maxlevel;

	gcomprisGame->sublevel=0;
      }

      play_sound (SOUNDLISTFILE, "bonus");
      shapegame_next_level();
    }
  else
    play_sound (SOUNDLISTFILE, "crash");

}

static void destroy_shape (Shape *shape)
{
  g_free(shape->name);
  g_free(shape->pixmapfile);
  g_free(shape);
}

/* Destroy all the items */
static void shapegame_destroy_all_items()
{
  Shape *shape;

  printf(">shapegame_destroy_all_items\n");

  /* Cleanup of the shapes */
  while(g_list_length(shape_list)>0) 
    {
      shape = g_list_nth_data(shape_list, 0);
      shape_list = g_list_remove (shape_list, shape);
      destroy_shape(shape);
    }
  
  if (shapelist_table)
    {
      /* Deleting the root item automatically deletes children items */
      gtk_object_destroy (GTK_OBJECT(shape_list_root_item));
      gtk_object_destroy (GTK_OBJECT(shape_root_item));

      g_hash_table_destroy (shapelist_table);
      shapelist_table=NULL;
    }
}

static GnomeCanvasItem *shapegame_init_canvas(GnomeCanvasGroup *parent)
{
  GnomeCanvasItem *item;


  shape_list_root_item = \
    gnome_canvas_item_new (parent,
			   gnome_canvas_group_get_type (),
			   "x", (double)gcomprisGame->width/SHAPE_BOX_WIDTH_RATIO,
			   "y", (double)0,
			   NULL);
  item_list = g_list_append (item_list, shape_list_root_item);

  /* Create the background box */
  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_list_root_item),
				gnome_canvas_rect_get_type (),
				"x1", (double)-gcomprisGame->width/SHAPE_BOX_WIDTH_RATIO,
				"y1", (double)0,
				"x2", (double)gcomprisGame->width/SHAPE_BOX_WIDTH_RATIO,
				"y2", (double)gcomprisGame->height-2,
				"fill_color_rgba", 0x5f9ea080,
				"outline_color", "cyan",
				"width_units", (double)1,
				NULL);
  item_list = g_list_append (item_list, item);

  shape_root_item = \
    gnome_canvas_item_new (parent,
			   gnome_canvas_group_get_type (),
			   "x", (double)gcomprisGame->width/SHAPE_BOX_WIDTH_RATIO,
			   "y", (double)0,
			   NULL);
  item_list = g_list_append (item_list, shape_root_item);

  return (item);
}

/**************************************************************
 * Shape list management
 */

/* Add the given shape to the list of shapes
 * Do nothing if the shape is already in
 */
static void
add_shape_to_list_of_shapes(Shape *shape)
{
  GnomeCanvasItem *item;
  gint MAX_NUMBER_OF_SHAPES = 6;
  double SHAPE_ELEM_NUMBER_IN_HEIGHT = (double)gcomprisGame->height/MAX_NUMBER_OF_SHAPES;

  if(!shapelist_table)
    shapelist_table= g_hash_table_new (g_str_hash, g_str_equal);

  /* This pixmap is not yet in the list of shapes */
  if(g_hash_table_lookup (shapelist_table, shape->pixmapfile)==NULL)
    {
      double y_offset = 0;
      GdkImlibImage *pixmap = NULL;

      y_offset = SHAPE_ELEM_NUMBER_IN_HEIGHT/2 +
	g_hash_table_size(shapelist_table)*SHAPE_ELEM_NUMBER_IN_HEIGHT;

      /* So this shape is not yet in, let's put it in now */
      g_hash_table_insert (shapelist_table, shape->pixmapfile, shape);

      pixmap = load_pixmap(shape->pixmapfile);
      if(pixmap)
	{
	  double w, h;
	  guint SHAPE_SIZE = SHAPE_ELEM_NUMBER_IN_HEIGHT - 20;
	  Shape *icon_shape;

	  /* Calc a zoom factor so that the shape will fit in the shapelist
	     whatever its current size */
	  w = SHAPE_SIZE;
	  h = pixmap->rgb_height * ( w / pixmap->rgb_width);

	  if(h > SHAPE_SIZE)
	    {
	      h = SHAPE_SIZE;
	      w = pixmap->rgb_width * ( h / pixmap->rgb_height);
	    }

	  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_list_root_item),
					gnome_canvas_image_get_type (),
					"image", pixmap, 
					"x", (double)0,
					"y", (double)y_offset,
					"width", (double) w,
					"height", (double) h,
					"anchor", GTK_ANCHOR_CENTER,
					NULL);

	  icon_shape = create_shape(SHAPE_ICON, shape->name, shape->pixmapfile, 
				    (double)0, (double)y_offset, 
				    (double)1, (double)1,
				    0);
	  icon_shape->item = item;

	  setup_item(item, icon_shape);
	  item_list = g_list_append (item_list, item);
	}
    }
}

/*
 * Find the closest shape from the given point if it is located at
 * a distance under the given limit.
 */
static Shape *find_closest_shape(double x, double y, double limit)
{
  GList *list;
  double goodDist = limit;
  Shape *candidateShape = NULL;

  /* loop through all our shapes */
  for(list = shape_list; list != NULL; list = list->next) {
    Shape *shape = list->data;
    double dist;

    if(shape->type==SHAPE_TARGET)
      {
	/* Calc the distance between this shape and the given coord */
	dist = sqrtf(powf((shape->x-x),2)*powf((shape->y-y),2));
	
	if(dist<goodDist)
	  {
	    goodDist=dist;
	    candidateShape=shape;
	  }
      }
  }

  return candidateShape;
}

#if 0
static void dump_shape(Shape *shape)
{
  printf("dump_shape name=%s found=%d type=%d ", shape->name, shape->found, shape->type);
  if(shape->bad_item)
    printf("bad_item=TRUE ");
  if(shape->icon_shape)
    printf("icon_shape=%s", shape->icon_shape->name);
  printf("\n");
}
#endif

/*
 * Given the shape, if it has an icon, it puts it back to
 * the list of shapes
 */
static void shape_goes_back_to_list(Shape *shape, GnomeCanvasItem *item)
{
  if(shape->icon_shape!=NULL)
    {
      /* There was a previous icon here, put it back to the list */
      gnome_canvas_item_move(shape->icon_shape->item, 
			     shape->icon_shape->x - shape->x,
			     shape->icon_shape->y - shape->y);
      gnome_canvas_item_show(shape->icon_shape->item);
      shape->icon_shape=NULL;

      gnome_canvas_item_hide(item);
      play_sound (SOUNDLISTFILE, "gobble");
    }
}

static gint
item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
{
   static double x, y;
   static double offset_x, offset_y;
   double new_x, new_y;
   GdkCursor *fleur;
   static int dragging;
   double item_x, item_y;

   if(!game_running())
     return FALSE;

   if(shape==NULL) {
     g_warning("Shape is NULL : Should not happen");
     return FALSE;
   }

   item_x = event->button.x;
   item_y = event->button.y;
   gnome_canvas_item_w2i(item->parent, &item_x, &item_y);

   switch (event->type) 
     {
     case GDK_BUTTON_PRESS:
       switch(event->button.button) 
         {
         case 1:
           if (event->button.state & GDK_SHIFT_MASK)
             {
	       /* Cheat code to save an XML file */
	       write_xml_file("/tmp/gcompris-board.xml");
	     }
	   else
	     {
	       x = item_x;
	       y = item_y;
	       
	       switch(shape->type)
		 {
		 case SHAPE_TARGET:
		   gnome_canvas_item_hide(item);
		   if( shape->icon_shape!=NULL)
		     {
		       item = shape->icon_shape->item;
		       gnome_canvas_item_show(item);
		       shape->icon_shape=NULL;
		     }
		   break;
		 default:
		   break;
		 }
	       /* This records the offset between the mouse pointer and the grabbed item center */
	       offset_x = x - shape->x;
	       offset_y = y - shape->y;
	       
	       if(item==NULL)
		 return FALSE;
	       
	       fleur = gdk_cursor_new(GDK_FLEUR);
	       
	       gnome_canvas_item_raise_to_top(item);
	       gnome_canvas_item_grab(item,
				      GDK_POINTER_MOTION_MASK | 
				      GDK_BUTTON_RELEASE_MASK,
				      fleur,
				      event->button.time);
	       gdk_cursor_destroy(fleur);
	       dragging = TRUE;
	     }
	   break;
	   
         case 3:
	   /* If There was a previous icon here, put it back to the list */
	   shape_goes_back_to_list(shape, item);
	   shape->found = FALSE;
	   break;

         default:
           break;
         }
       break;

     case GDK_MOTION_NOTIFY:
       if (dragging && (event->motion.state & GDK_BUTTON1_MASK)) 
         {
	   new_x = item_x;
	   new_y = item_y;
	       
	   gnome_canvas_item_move(item, new_x - x, new_y - y);

	   x = new_x;
	   y = new_y;
         }
       break;
           
     case GDK_BUTTON_RELEASE:
       if(dragging) 
	 {
	   Shape *targetshape = NULL;

	   gnome_canvas_item_ungrab(item, event->button.time);
	   dragging = FALSE;

	   targetshape = find_closest_shape(item_x, item_y, 1000);
	   if(targetshape!=NULL)
	     {
	       /* Finish the placement of the grabbed item anyway */
	       gnome_canvas_item_move(item, 
				      targetshape->x - x + offset_x, 
				      targetshape->y - y + offset_y);

	       if(strcmp(targetshape->name, shape->name)==0)
		 {
		   /* You got it right perfect */
		   if(targetshape->bad_item!=NULL)
		     {
		       gnome_canvas_item_hide(targetshape->bad_item);
		       gtk_object_destroy (GTK_OBJECT(targetshape->bad_item));
		       targetshape->bad_item=NULL;
		     }
		   targetshape->found = TRUE;
		   gnome_canvas_item_show(targetshape->item);
		 }
	       else
		 {
		   /* Oups wrong position, create the wrong pixmap item */
		   GdkImlibImage *pixmap;
		   GnomeCanvasItem *item;

		   targetshape->found = FALSE;
		   gnome_canvas_item_hide(targetshape->item);

		   /* There is already a bad item, delete it */
		   if(targetshape->bad_item!=NULL)
		     gtk_object_destroy (GTK_OBJECT(targetshape->bad_item));

		   pixmap = load_pixmap(shape->pixmapfile);

		   item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
						 gnome_canvas_image_get_type (),
						 "image", pixmap, 
						 "x", (double)targetshape->x,
						 "y", (double)targetshape->y,
						 "width", (double) pixmap->rgb_width * targetshape->zoomx,
						 "height", (double) pixmap->rgb_height * targetshape->zoomy,
						 "anchor", GTK_ANCHOR_CENTER,
						 NULL);
		   targetshape->bad_item = item;
		   item_list = g_list_append (item_list, item);
		   setup_item(item, targetshape);
		 }

	       /* If There was a previous icon here, put it back to the list */
	       shape_goes_back_to_list(targetshape, item);

	       /* Set the icon item */
	       targetshape->icon_shape=shape;
	     }
	   else
	     {
	       /* The item has no close position */
	       switch(shape->type)
		 {
		 case SHAPE_TARGET:
		   gnome_canvas_item_hide(item);
		   if( shape->icon_shape!=NULL)
		     {
		       item = shape->icon_shape->item;
		       gnome_canvas_item_show(item);
		     }
		   break;
		 case SHAPE_ICON:
		   break;
		 default:
		   break;
		 }
	       /* Move back the item home */
	       gnome_canvas_item_move(item, 
				      shape->x - x + offset_x,
				      shape->y - y + offset_y);
	     }
	 }
       break;

     default:
       break;
     }
         
   return FALSE;
 }

static void
setup_item(GnomeCanvasItem *item, Shape *shape)
{
  gtk_signal_connect(GTK_OBJECT(item), "event",
		     (GtkSignalFunc) item_event,
		     shape);
}

/*
 * Thanks for George Lebl <jirka@5z.com> for his Genealogy example
 * for all the XML stuff there
 */

/* Adds a shape to the canvas */
static void
add_shape_to_canvas(Shape *shape)
{
  GdkImlibImage *pixmap;
  GnomeCanvasItem *item;

  /* checking of preconditions, this is usefull for tracking down bugs,
     but you should not count on these always being compiled in */
  g_return_if_fail(shape != NULL);
	
  pixmap = load_pixmap(shape->pixmapfile);
  if(pixmap)
    {	
      /* Display a point to display the target location of this shape */
      item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
				    gnome_canvas_ellipse_get_type(),
				    "x1", (double)shape->x-2.5,
				    "y1", (double)shape->y-2.5,
				    "x2", (double)shape->x+2.5,
				    "y2", (double)shape->y+2.5,
				    "fill_color_rgba", 0xEf000080,
				    "outline_color", "black",
				    "width_pixels", 0,
				    NULL);
      gnome_canvas_item_lower_to_bottom(item);
      item_list = g_list_append (item_list, item);

      /* Display the shape itself but hide it until the user puts the right shape on it */
      /* I have to do it this way for the positionning (lower/raise) complexity          */
      item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
				    gnome_canvas_image_get_type (),
				    "image", pixmap, 
				    "x", (double)shape->x,
				    "y", (double)shape->y,
				    "width", (double) pixmap->rgb_width * shape->zoomx,
				    "height", (double) pixmap->rgb_height * shape->zoomy,
				    "anchor", GTK_ANCHOR_CENTER,
				    NULL);

      gnome_canvas_item_lower(item, ABS(shape->position)); 
      gnome_canvas_item_hide(item);

      /* Associate this item to this shape */
      shape->item = item;

      setup_item(item, shape);
      item_list = g_list_append (item_list, item);

      add_shape_to_list_of_shapes(shape);
    }
}

void create_title(char *name, double x, double y, char *justification)
{
  GnomeCanvasItem *item;
  GdkFont *gdk_font;

  /* Load a gdk font */
  gdk_font = gdk_font_load ("-adobe-times-medium-r-normal--*-180-*-*-*-*-*-*");

  item = \
    gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
			   gnome_canvas_text_get_type (),
			   "text", name,
			   "font_gdk", gdk_font,
			   "x", x,
			   "y", y,
			   "anchor", GTK_ANCHOR_CENTER,
			   "justification", justification,
			   "fill_color", "green",
			   NULL);

}

static Shape *
create_shape(ShapeType type, char *name, char *pixmapfile, double x, double y, double zoomx, 
			   double zoomy, guint position)
{
  Shape *shape;
  
  /* allocate a new shape */
  shape = g_new(Shape,1);
  
  shape->name = g_strdup(name);
  shape->pixmapfile = g_strdup(pixmapfile);
  shape->x = x;
  shape->y = y;
  shape->zoomx = zoomx;
  shape->zoomy = zoomy;
  shape->position = position;
  shape->type = type;

  shape->bad_item = NULL;
  shape->icon_shape = NULL;

  shape->found = FALSE;

  /* add the shape to the list */
  shape_list = g_list_append(shape_list, shape);
  
  return shape;
}


static void
add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
{
  char *name, *pixmapfile, *cx, *cy, *czoomx, *czoomy, *cposition, *ctype, *justification;
  double x, y, zoomx, zoomy;
  guint position;
  ShapeType type;
  Shape *shape;

  if(/* if the node has no name */
     !xmlnode->name ||
     /* or if the name is not "Shape" */
     (g_strcasecmp(xmlnode->name,"Shape")!=0 &&
     /* or if the name is not "Title" */
      g_strcasecmp(xmlnode->name,"Title")!=0)
     )
    return;
  
  /* get the name of the shape */
  name = xmlGetProp(xmlnode,"name");

  pixmapfile = xmlGetProp(xmlnode,"pixmapfile");
  /* if unspecified, make it "Unknown" */
  if(!pixmapfile) pixmapfile = "Unknown";

  /* get the X coord of the shape */
  cx = xmlGetProp(xmlnode,"x");
  if(!cx) cx = "100";
  x = atof(cx);

  /* get the Y coord of the shape */
  cy = xmlGetProp(xmlnode,"y");
  if(!cy) cy = "100";
  y = atof(cy);

  /* get the ZOOMX coord of the shape */
  czoomx = xmlGetProp(xmlnode,"zoomx");
  if(!czoomx) czoomx = "1";
  zoomx = atof(czoomx);

  /* get the ZOOMY coord of the shape */
  czoomy = xmlGetProp(xmlnode,"zoomy");
  if(!czoomy) czoomy = "1";
  zoomy = atof(czoomy);

  /* get the POSITION coord of the shape */
  cposition = xmlGetProp(xmlnode,"position");
  if(!cposition) cposition = "0";
  position = atoi(cposition);

  /* get the TYPE of the shape */
  ctype = xmlGetProp(xmlnode,"type");
  if(!ctype) ctype = "1"; /* SHAPE_TARGET is default */
  type = (ShapeType)atoi(ctype);

  /* get the JUSTIFICATION of the Title */
  justification = xmlGetProp(xmlnode,"justification");
  if(!justification) justification = "GTK_JUSTIFICATION_CENTER"; /* GTK_JUSTIFICATION_CENTER is default */


  if(g_strcasecmp(xmlnode->name,"Shape")==0)
    {
      /* add the shape to the database */
      shape = create_shape(type, name, pixmapfile, x, y, zoomx, zoomy, position);
      add_shape_to_canvas(shape);
    } 
  else if (g_strcasecmp(xmlnode->name,"Title")==0)
    {
      create_title(name, x, y, justification);
    }
  
}

/* parse the doc, add it to our internal structures and to the clist */
static void
parse_doc(xmlDocPtr doc)
{
  xmlNodePtr node;
  
  /* find <Shape> nodes and add them to the list, this just
     loops through all the children of the root of the document */
  for(node = doc->root->childs; node != NULL; node = node->next) {
    /* add the shape to the list, there are no children so
       we pass NULL as the node of the child */
    add_xml_shape_to_data(node,NULL);
  }
}



/* read an xml file into our memory structures and update our view,
   dump any old data we have in memory if we can load a new set */
static gboolean
read_xml_file(char *fname)
{
  /* pointer to the new doc */
  xmlDocPtr doc;
  
  g_return_val_if_fail(fname!=NULL,FALSE);
  
  /* if the file doesn't exist */
  if(!g_file_exists(fname)) 
    {
      g_warning(_("Couldn't find file %s !"), fname);
      return FALSE;
    }

  /* parse the new file and put the result into newdoc */
  doc = xmlParseFile(fname);
  /* in case something went wrong */
  if(!doc)
    return FALSE;
  
  if(/* if there is no root element */
     !doc->root ||
     /* if it doesn't have a name */
     !doc->root->name ||
     /* if it isn't a ShapeGame node */
     g_strcasecmp(doc->root->name,"ShapeGame")!=0) {
    xmlFreeDoc(doc);
    return FALSE;
  }
  
  /* parse our document and replace old data */
  parse_doc(doc);
  
  xmlFreeDoc(doc);
  
  return TRUE;
}

/* add a shape to xmlnode from node */
static void
write_shape_to_xml(xmlNodePtr xmlnode, Shape *shape)
{
  xmlNodePtr newxml;
  gchar *tmp;

  g_return_if_fail(xmlnode!=NULL);
  g_return_if_fail(shape!=NULL);
  
  /* make a new xml node (as a child of xmlnode) with an
     empty content */
  newxml = xmlNewChild(xmlnode,NULL,"Shape",NULL);
  /* set properties on it */
  xmlSetProp(newxml,"name",shape->name);
  xmlSetProp(newxml,"pixmapfile",shape->pixmapfile);

  tmp = g_strdup_printf("%f", shape->x);
  xmlSetProp(newxml,"x",tmp);
  g_free(tmp);

  tmp = g_strdup_printf("%f", shape->y);
  xmlSetProp(newxml,"y",tmp);
  g_free(tmp);

  tmp = g_strdup_printf("%f", shape->zoomx);
  xmlSetProp(newxml,"zoomx",tmp);
  g_free(tmp);

  tmp = g_strdup_printf("%f", shape->zoomy);
  xmlSetProp(newxml,"zoomy",tmp);
  g_free(tmp);

  tmp = g_strdup_printf("%d", shape->position);
  xmlSetProp(newxml,"position",tmp);
  g_free(tmp);

}

/* write an xml file 
 * This is used only for creating shapegame boards
 */
static gboolean
write_xml_file(char *fname)
{
  /* pointer to the new doc */
  xmlDocPtr doc;
  GList *list;
  
  g_return_val_if_fail(fname!=NULL,FALSE);
  
  /* create new xml document with version 1.0 */
  doc = xmlNewDoc("1.0");
  /* create a new root node "ShapeGame" */
  doc->root = xmlNewDocNode(doc, NULL, "ShapeGame", NULL);
  
  /* loop through all our shapes */
  for(list = shape_list; list != NULL; list = list->next) {
    Shape *shape = list->data;
    if(shape->type!=SHAPE_ICON)
      write_shape_to_xml(doc->root, shape);
  }
  
  /* try to save the file */
  if(xmlSaveFile(fname,doc) == -1) {
    xmlFreeDoc(doc);
    return FALSE;
  }
  
  xmlFreeDoc(doc);
  
  return TRUE;
}



/* Local Variables: */
/* mode:c */
/* eval:(load-library "time-stamp") */
/* eval:(make-local-variable 'write-file-hooks) */
/* eval:(add-hook 'write-file-hooks 'time-stamp) */
/* eval:(setq time-stamp-format '(time-stamp-yyyy/mm/dd time-stamp-hh:mm:ss user-login-name)) */
/* End: */
